/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.desktop.app;

import com.vektorsoft.demux.core.resources.LocaleChangeListener;
import com.vektorsoft.demux.core.mva.AbstractViewManager;
import com.vektorsoft.demux.core.mva.DMXAdapter;
import com.vektorsoft.demux.core.mva.DMXView;
import com.vektorsoft.demux.core.task.GUIBlockingScope;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import javafx.application.Platform;
import javafx.scene.Node;
import javafx.scene.layout.Pane;
import javafx.stage.Stage;

/**
 * Implementation of {@link com.vektorsoft.demux.core.mva.DMXViewManager} used for JavaFX applications.
 *
 * @author Vladimir Djurovic
 */
public class JFXViewManager extends AbstractViewManager {
    
    /** View ID of the root view. */
    public static final String JFX_ROOT_VIEW_ID = "com.vektorsoft.demux.desktop.jfxRootView";
    
    /** JavaFX stage. */
    private Stage stage;
    
    /**
     * Creates new instance of this view manager.
     * 
     * @param stage JavaFX stage
     * @param adapter adapter instance
     */
    public JFXViewManager(Stage stage, DMXAdapter adapter){
        this.stage = stage;
        this.adapter = adapter;
        JFXRootView rootView = new JFXRootView(this.stage);
        registerView(rootView);
    }

     @Override
    public void selectView(String viewId, Map dataValues) {
        System.out.println("Select view caled...");
    }

    @Override
    public void updateView(String viewId, Map dataValues) {
        getView(viewId).updateFromModel(dataValues);
        getView(viewId).render();
    }

    @Override
    public void updateViewData(final Map<String, Object> dataMap) {
        //find views that need to be refreshed
        final Set<String> viewIds = new HashSet<>();
        for(String id : dataMap.keySet()){
            viewIds.addAll(getViewIdsForData(id));
        }
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                for(String id : viewIds){
                    updateView(id, dataMap);
                }
            }
        });
    }

    @Override
    public final void registerView(final DMXView view) {
        super.registerView(view);
       
        // add view to window
        final String parentId = view.getParentViewId();
        if(parentId != null && !parentId.isEmpty()){
            Platform.runLater(new Runnable() {

                @Override
                public void run() {
                    DMXView parent = getView(parentId);
                    if(parent != null){
                        parent.addChildView(view);
                        view.render();
                    }
                    
                }
            });
            
        }
    }

    @Override
    protected void processDeferredViews(final String parentId, final Set<String> childrenIds) {
        if(childrenIds != null && !childrenIds.isEmpty()){
            for(final String id : childrenIds){
                Platform.runLater(new Runnable() {

                    @Override
                    public void run() {
                        DMXView view = getView(id);
                        getView(parentId).addChildView(view);
                        view.render();
                    }
                });
            }
        }
    }

    /**
     * This method will update GUI based on locale change.
     */
    @Override
    public void handleLocaleChange() {
        for(final LocaleChangeListener listener : localeListeners){
            Platform.runLater(new Runnable() {

                @Override
                public void run() {
                    listener.onLocaleChange();
                    if(listener instanceof DMXView){
                        ((DMXView)listener).render();
                    }
                }
            });
        }
    }

    @Override
    public void blockGui(boolean state, GUIBlockingScope scope, String... ids) {
        switch(scope){
            case TOP_LEVEL:
                // not implemented
                break;
            case COMPONENTS:
                for(String id : ids){
                    Node n = stage.getScene().lookup("#" + id);
                    if(n != null){
                        n.setDisable(state);
                    }
                }
                break;
        }
    }
    
    
    /**
     * <p>
     * This class represents root view of JavaFX application. All views added to application are rooted in this view.
     * </p>
     * <p>
     *  This view does not have any data associated, but merely serves as a container for other views. When multiple subviews are added
     * to this views,they will be stacked on top of each other, with the last view being on top.
     * </p>
     */
    private static  class JFXRootView implements DMXView {
        
        /** Root node of JavaFX scene graph. */
        private Pane root;

        /**
         * Create new instance.
         * 
         * @param stage JavaFX stage
         */
        JFXRootView(Stage stage) {
            root = (Pane)stage.getScene().getRoot();
        }
        
        

        @Override
        public void render() {
            // do nothing
        }

        @Override
        public String getViewId() {
            return JFX_ROOT_VIEW_ID;
        }

        @Override
        public String getParentViewId() {
            return null;
        }

        @Override
        public Object viewPlacementConstraint() {
             return null;
        }

        @Override
        public void addChildView(DMXView child) {
            if(!(child.getViewUI() instanceof Node)){
                throw new IllegalArgumentException("View must be instance of javafx.scene.Node");
            }
            root.getChildren().add((Node)child.getViewUI());
        }

        @Override
        public Set<String> getViewDataIds() {
            return Collections.EMPTY_SET;
        }

        @Override
        public void updateFromModel(Map<String, Object> dataMap) {
            
        }

        @Override
        public Object getViewUI() {
            return root;
        }
        
    }
}
